@@ -21,6 +21,8 @@ module Statistics.Distribution
     , Mean(..)
     , MaybeVariance(..)
     , Variance(..)
+    , MaybeEntropy(..)
+    , Entropy(..)
       -- ** Random number generation
     , ContGen(..)
     , DiscreteGen(..)
@@ -30,12 +32,13 @@ module Statistics.Distribution
     , sumProbabilities
     ) where
 
-import Control.Applicative     ((<$>), Applicative(..))
+import Control.Applicative ((<$>), Applicative(..))
 import Control.Monad.Primitive (PrimMonad,PrimState)
-
+import Prelude hiding (sum)
+import Statistics.Function (square)
+import Statistics.Sample.Internal (sum)
+import System.Random.MWC (Gen, uniform)
 import qualified Data.Vector.Unboxed as U
-import System.Random.MWC
-
 
 
 -- | Type class common to all distributions. Only c.d.f. could be
@@ -43,17 +46,21 @@ import System.Random.MWC
 class Distribution d where
     -- | Cumulative distribution function.  The probability that a
     -- random variable /X/ is less or equal than /x/,
-    -- i.e. P(/X/&#8804;/x/). 
+    -- i.e. P(/X/&#8804;/x/). Cumulative should be defined for
+    -- infinities as well:
+    --
+    -- > cumulative d +∞ = 1
+    -- > cumulative d -∞ = 0
     cumulative :: d -> Double -> Double
 
     -- | One's complement of cumulative distibution:
     --
     -- > complCumulative d x = 1 - cumulative d x
     --
-    -- It's useful when one is interested in P(/X/&#8805;/x/) and
+    -- It's useful when one is interested in P(/X/</x/) and
     -- expression on the right side begin to lose precision. This
     -- function have default implementation but implementors are
-    -- encouraged to provide more precise implementation
+    -- encouraged to provide more precise implementation.
     complCumulative :: d -> Double -> Double
     complCumulative d x = 1 - cumulative d x
 
@@ -61,20 +68,32 @@ class Distribution d where
 class Distribution  d => DiscreteDistr d where
     -- | Probability of n-th outcome.
     probability :: d -> Int -> Double
+    probability d = exp . logProbability d
+
+    -- | Logarithm of probability of n-th outcome
+    logProbability :: d -> Int -> Double
+    logProbability d = log . probability d
 
 
--- | Continuous probability distributuion
+-- | Continuous probability distributuion.
+--
+--   Minimal complete definition is 'quantile' and either 'density' or
+--   'logDensity'.
 class Distribution d => ContDistr d where
     -- | Probability density function. Probability that random
     -- variable /X/ lies in the infinitesimal interval
     -- [/x/,/x+/&#948;/x/) equal to /density(x)/&#8901;&#948;/x/
     density :: d -> Double -> Double
+    density d = exp . logDensity d
 
     -- | Inverse of the cumulative distribution function. The value
     -- /x/ for which P(/X/&#8804;/x/) = /p/. If probability is outside
     -- of [0,1] range function should call 'error'
     quantile :: d -> Double -> Double
 
+    -- | Natural logarithm of density.
+    logDensity :: d -> Double -> Double
+    logDensity d = log . density d
 
 
 -- | Type class for distributions with mean. 'maybeMean' should return
@@ -108,10 +127,26 @@ class MaybeMean d => MaybeVariance d where
 --   Minimal complete definition is 'variance' or 'stdDev'
 class (Mean d, MaybeVariance d) => Variance d where
     variance :: d -> Double
-    variance d = x * x where x = stdDev d
+    variance d = square (stdDev d)
     stdDev   :: d -> Double
     stdDev = sqrt . variance
 
+-- | Type class for distributions with entropy, meaning Shannon entropy
+--   in the case of a discrete distribution, or differential entropy in the
+--   case of a continuous one.  'maybeEntropy' should return 'Nothing' if
+--   entropy is undefined for the chosen parameter values.
+class (Distribution d) => MaybeEntropy d where
+  -- | Returns the entropy of a distribution, in nats, if such is defined.
+  maybeEntropy :: d -> Maybe Double
+
+-- | Type class for distributions with entropy, meaning Shannon
+--   entropy in the case of a discrete distribution, or differential
+--   entropy in the case of a continuous one.  If the distribution has
+--   well-defined entropy for all valid parameter values then it
+--   should be an instance of this type class.
+class (MaybeEntropy d) => Entropy d where
+  -- | Returns the entropy of a distribution, in nats.
+  entropy :: d -> Double
 
 -- | Generate discrete random variates which have given
 --   distribution.
@@ -130,7 +165,6 @@ genContinous :: (ContDistr d, PrimMonad m) => d -> Gen (PrimState m) -> m Double
 genContinous d gen = do
   x <- uniform gen
   return $! quantile d x
-{-# INLINE genContinous #-}
 
 data P = P {-# UNPACK #-} !Double {-# UNPACK #-} !Double
 
@@ -140,7 +174,7 @@ data P = P {-# UNPACK #-} !Double {-# UNPACK #-} !Double
 -- bisection with the given guess as a starting point.  The upper and
 -- lower bounds specify the interval in which the probability
 -- distribution reaches the value /p/.
-findRoot :: ContDistr d => 
+findRoot :: ContDistr d =>
             d                   -- ^ Distribution
          -> Double              -- ^ Probability /p/
          -> Double              -- ^ Initial guess
@@ -169,7 +203,6 @@ findRoot d prob = loop 0 1
 -- | Sum probabilities in inclusive interval.
 sumProbabilities :: DiscreteDistr d => d -> Int -> Int -> Double
 sumProbabilities d low hi =
-  -- Return value is forced to be less than 1 to guard againist roundoff errors. 
+  -- Return value is forced to be less than 1 to guard againist roundoff errors.
   -- ATTENTION! this check should be removed for testing or it could mask bugs.
-  min 1 . U.sum . U.map (probability d) $ U.enumFromTo low hi
-{-# INLINE sumProbabilities #-}
\ No newline at end of file
+  min 1 . sum . U.map (probability d) $ U.enumFromTo low hi
