@@ -1,4 +1,4 @@
-{-# LANGUAGE DeriveDataTypeable #-}
+{-# LANGUAGE DeriveDataTypeable, DeriveGeneric #-}
 -- |
 -- Module    : Statistics.Distribution.ChiSquared
 -- Copyright : (c) 2010 Alexey Khudyakov
@@ -18,30 +18,40 @@ module Statistics.Distribution.ChiSquared (
         , chiSquaredNDF
         ) where
 
-import Data.Typeable         (Typeable)
-import Numeric.SpecFunctions (incompleteGamma,invIncompleteGamma,logGamma)
+import Data.Aeson (FromJSON, ToJSON)
+import Data.Binary (Binary)
+import Data.Data (Data, Typeable)
+import GHC.Generics (Generic)
+import Numeric.SpecFunctions (
+  incompleteGamma,invIncompleteGamma,logGamma,digamma)
 
 import qualified Statistics.Distribution         as D
 import qualified System.Random.MWC.Distributions as MWC
+import Data.Binary (put, get)
 
 
 -- | Chi-squared distribution
 newtype ChiSquared = ChiSquared Int
-                     deriving (Show,Typeable)
+                     deriving (Eq, Read, Show, Typeable, Data, Generic)
+
+instance FromJSON ChiSquared
+instance ToJSON ChiSquared
+
+instance Binary ChiSquared where
+    get = fmap ChiSquared get
+    put (ChiSquared x) = put x
 
 -- | Get number of degrees of freedom
 chiSquaredNDF :: ChiSquared -> Int
 chiSquaredNDF (ChiSquared ndf) = ndf
-{-# INLINE chiSquaredNDF #-}
 
 -- | Construct chi-squared distribution. Number of degrees of freedom
 --   must be positive.
 chiSquared :: Int -> ChiSquared
 chiSquared n
-  | n <= 0    = error $ 
+  | n <= 0    = error $
      "Statistics.Distribution.ChiSquared.chiSquared: N.D.F. must be positive. Got " ++ show n
   | otherwise = ChiSquared n
-{-# INLINE chiSquared #-}
 
 instance D.Distribution ChiSquared where
   cumulative = cumulative
@@ -52,11 +62,9 @@ instance D.ContDistr ChiSquared where
 
 instance D.Mean ChiSquared where
     mean (ChiSquared ndf) = fromIntegral ndf
-    {-# INLINE mean #-}
 
 instance D.Variance ChiSquared where
     variance (ChiSquared ndf) = fromIntegral (2*ndf)
-    {-# INLINE variance #-}
 
 instance D.MaybeMean ChiSquared where
     maybeMean = Just . D.mean
@@ -65,6 +73,17 @@ instance D.MaybeVariance ChiSquared where
     maybeStdDev   = Just . D.stdDev
     maybeVariance = Just . D.variance
 
+instance D.Entropy ChiSquared where
+  entropy (ChiSquared ndf) =
+    let kHalf = 0.5 * fromIntegral ndf in
+    kHalf
+    + log 2
+    + logGamma kHalf
+    + (1-kHalf) * digamma kHalf
+
+instance D.MaybeEntropy ChiSquared where
+  maybeEntropy = Just . D.entropy
+
 instance D.ContGen ChiSquared where
     genContVar (ChiSquared n) = MWC.chiSquare n
 
@@ -75,7 +94,6 @@ cumulative chi x
   | otherwise = incompleteGamma (ndf/2) (x/2)
   where
     ndf = fromIntegral $ chiSquaredNDF chi
-{-# INLINE cumulative #-}
 
 density :: ChiSquared -> Double -> Double
 density chi x
@@ -85,7 +103,6 @@ density chi x
     ndf  = fromIntegral $ chiSquaredNDF chi
     ndf2 = ndf/2
     x2   = x/2
-{-# INLINE density #-}
 
 quantile :: ChiSquared -> Double -> Double
 quantile (ChiSquared ndf) p
@@ -94,4 +111,3 @@ quantile (ChiSquared ndf) p
   | p > 0 && p < 1 = 2 * invIncompleteGamma (fromIntegral ndf / 2) p
   | otherwise      =
     error $ "Statistics.Distribution.ChiSquared.quantile: p must be in [0,1] range. Got: "++show p
-{-# INLINE quantile #-}
