@@ -1,4 +1,4 @@
-{-# LANGUAGE DeriveDataTypeable #-}
+{-# LANGUAGE DeriveDataTypeable, DeriveGeneric #-}
 -- |
 -- Module    : Statistics.Distribution.Exponential
 -- Copyright : (c) 2009 Bryan O'Sullivan
@@ -23,31 +23,47 @@ module Statistics.Distribution.Exponential
     , edLambda
     ) where
 
-import Data.Typeable (Typeable)
+import Data.Aeson (FromJSON, ToJSON)
+import Data.Binary (Binary)
+import Data.Data (Data, Typeable)
+import GHC.Generics (Generic)
+import Numeric.MathFunctions.Constants (m_neg_inf)
 import qualified Statistics.Distribution         as D
 import qualified Statistics.Sample               as S
 import qualified System.Random.MWC.Distributions as MWC
 import Statistics.Types (Sample)
+import Data.Binary (put, get)
+
 
 newtype ExponentialDistribution = ED {
       edLambda :: Double
-    } deriving (Eq, Read, Show, Typeable)
+    } deriving (Eq, Read, Show, Typeable, Data, Generic)
+
+instance FromJSON ExponentialDistribution
+instance ToJSON ExponentialDistribution
+
+instance Binary ExponentialDistribution where
+    put = put . edLambda
+    get = fmap ED get
 
 instance D.Distribution ExponentialDistribution where
     cumulative      = cumulative
     complCumulative = complCumulative
 
 instance D.ContDistr ExponentialDistribution where
-    density  = density
+    density (ED l) x
+      | x < 0     = 0
+      | otherwise = l * exp (-l * x)
+    logDensity (ED l) x
+      | x < 0     = m_neg_inf
+      | otherwise = log l + (-l * x)
     quantile = quantile
 
 instance D.Mean ExponentialDistribution where
     mean (ED l) = 1 / l
-    {-# INLINE mean #-}
 
 instance D.Variance ExponentialDistribution where
     variance (ED l) = 1 / (l * l)
-    {-# INLINE variance #-}
 
 instance D.MaybeMean ExponentialDistribution where
     maybeMean = Just . D.mean
@@ -56,23 +72,23 @@ instance D.MaybeVariance ExponentialDistribution where
     maybeStdDev   = Just . D.stdDev
     maybeVariance = Just . D.variance
 
+instance D.Entropy ExponentialDistribution where
+  entropy (ED l) = 1 - log l
+
+instance D.MaybeEntropy ExponentialDistribution where
+  maybeEntropy = Just . D.entropy
+
 instance D.ContGen ExponentialDistribution where
   genContVar = MWC.exponential . edLambda
 
 cumulative :: ExponentialDistribution -> Double -> Double
 cumulative (ED l) x | x <= 0    = 0
                     | otherwise = 1 - exp (-l * x)
-{-# INLINE cumulative #-}
 
 complCumulative :: ExponentialDistribution -> Double -> Double
 complCumulative (ED l) x | x <= 0    = 1
                          | otherwise = exp (-l * x)
-{-# INLINE complCumulative #-}
 
-density :: ExponentialDistribution -> Double -> Double
-density (ED l) x | x < 0     = 0
-                 | otherwise = l * exp (-l * x)
-{-# INLINE density #-}
 
 quantile :: ExponentialDistribution -> Double -> Double
 quantile (ED l) p
@@ -80,19 +96,16 @@ quantile (ED l) p
   | p >= 0 && p < 1 = -log (1 - p) / l
   | otherwise       =
     error $ "Statistics.Distribution.Exponential.quantile: p must be in [0,1] range. Got: "++show p
-{-# INLINE quantile #-}
 
 -- | Create an exponential distribution.
 exponential :: Double            -- ^ &#955; (scale) parameter.
             -> ExponentialDistribution
 exponential l
-  | l <= 0 = 
+  | l <= 0 =
     error $ "Statistics.Distribution.Exponential.exponential: scale parameter must be positive. Got " ++ show l
   | otherwise = ED l
-{-# INLINE exponential #-}
 
 -- | Create exponential distribution from sample. No tests are made to
 -- check whether it truly is exponential.
 exponentialFromSample :: Sample -> ExponentialDistribution
 exponentialFromSample = ED . S.mean
-{-# INLINE exponentialFromSample #-}
