@@ -1,4 +1,4 @@
-{-# LANGUAGE DeriveDataTypeable #-}
+{-# LANGUAGE DeriveDataTypeable, DeriveGeneric #-}
 -- |
 -- Module    : Statistics.Distribution.Hypergeometric
 -- Copyright : (c) 2009 Bryan O'Sullivan
@@ -27,15 +27,28 @@ module Statistics.Distribution.Hypergeometric
     , hdK
     ) where
 
-import Data.Typeable         (Typeable)
+import Data.Aeson (FromJSON, ToJSON)
+import Data.Binary (Binary)
+import Data.Data (Data, Typeable)
+import GHC.Generics (Generic)
+import Numeric.MathFunctions.Constants (m_epsilon)
 import Numeric.SpecFunctions (choose)
 import qualified Statistics.Distribution as D
+import Data.Binary (put, get)
+import Control.Applicative ((<$>), (<*>))
 
 data HypergeometricDistribution = HD {
       hdM :: {-# UNPACK #-} !Int
     , hdL :: {-# UNPACK #-} !Int
     , hdK :: {-# UNPACK #-} !Int
-    } deriving (Eq, Read, Show, Typeable)
+    } deriving (Eq, Read, Show, Typeable, Data, Generic)
+
+instance FromJSON HypergeometricDistribution
+instance ToJSON HypergeometricDistribution
+
+instance Binary HypergeometricDistribution where
+    get = HD <$> get <*> get <*> get
+    put (HD x y z) = put x >> put y >> put z
 
 instance D.Distribution HypergeometricDistribution where
     cumulative = cumulative
@@ -56,7 +69,11 @@ instance D.MaybeVariance HypergeometricDistribution where
     maybeStdDev   = Just . D.stdDev
     maybeVariance = Just . D.variance
 
+instance D.Entropy HypergeometricDistribution where
+  entropy = directEntropy
 
+instance D.MaybeEntropy HypergeometricDistribution where
+  maybeEntropy = Just . D.entropy
 
 variance :: HypergeometricDistribution -> Double
 variance (HD m l k) = (k' * ml) * (1 - ml) * (l' - k') / (l' - 1)
@@ -64,11 +81,17 @@ variance (HD m l k) = (k' * ml) * (1 - ml) * (l' - k') / (l' - 1)
         l' = fromIntegral l
         k' = fromIntegral k
         ml = m' / l'
-{-# INLINE variance #-}
 
 mean :: HypergeometricDistribution -> Double
 mean (HD m l k) = fromIntegral k * fromIntegral m / fromIntegral l
-{-# INLINE mean #-}
+
+directEntropy :: HypergeometricDistribution -> Double
+directEntropy d@(HD m _ _) =
+    negate . sum $
+  takeWhile (< negate m_epsilon) $
+  dropWhile (not . (< negate m_epsilon)) $
+  [ let x = probability d n in x * log x | n <- [0..m]]
+
 
 hypergeometric :: Int               -- ^ /m/
                -> Int               -- ^ /l/
@@ -81,7 +104,6 @@ hypergeometric m l k
   | otherwise = HD m l k
     where
       msg = "Statistics.Distribution.Hypergeometric.hypergeometric: "
-{-# INLINE hypergeometric #-}
 
 -- Naive implementation
 probability :: HypergeometricDistribution -> Int -> Double
@@ -89,14 +111,15 @@ probability (HD mi li ki) n
   | n < max 0 (mi+ki-li) || n > min mi ki = 0
   | otherwise =
       choose mi n * choose (li - mi) (ki - n) / choose li ki
-{-# INLINE probability #-}
 
 cumulative :: HypergeometricDistribution -> Double -> Double
 cumulative d@(HD mi li ki) x
-  | n <  minN = 0 
-  | n >= maxN = 1
-  | otherwise = D.sumProbabilities d minN n
-    where
-      n    = floor x
-      minN = max 0 (mi+ki-li)
-      maxN = min mi ki
\ No newline at end of file
+  | isNaN x      = error "Statistics.Distribution.Hypergeometric.cumulative: NaN argument"
+  | isInfinite x = if x > 0 then 1 else 0
+  | n <  minN    = 0
+  | n >= maxN    = 1
+  | otherwise    = D.sumProbabilities d minN n
+  where
+    n    = floor x
+    minN = max 0 (mi+ki-li)
+    maxN = min mi ki
