@@ -1,4 +1,5 @@
-{-# LANGUAGE DeriveDataTypeable, OverloadedStrings, RecordWildCards #-}
+{-# LANGUAGE DeriveDataTypeable, DeriveGeneric, OverloadedStrings,
+    RecordWildCards #-}
 
 -- |
 -- Module    : Statistics.Resampling.Bootstrap
@@ -20,18 +21,24 @@ module Statistics.Resampling.Bootstrap
     -- $references
     ) where
 
+import Control.Applicative ((<$>), (<*>))
 import Control.DeepSeq (NFData)
 import Control.Exception (assert)
-import Control.Monad.Par (runPar, parMap)
+import Control.Monad.Par (parMap, runPar)
+import Data.Aeson (FromJSON, ToJSON)
+import Data.Binary (Binary)
+import Data.Binary (put, get)
 import Data.Data (Data)
 import Data.Typeable (Typeable)
 import Data.Vector.Unboxed ((!))
+import GHC.Generics (Generic)
 import Statistics.Distribution (cumulative, quantile)
 import Statistics.Distribution.Normal
 import Statistics.Resampling (Resample(..), jackknife)
 import Statistics.Sample (mean)
 import Statistics.Types (Estimator, Sample)
 import qualified Data.Vector.Unboxed as U
+import qualified Statistics.Resampling as R
 
 -- | A point and interval estimate computed via an 'Estimator'.
 data Estimate = Estimate {
@@ -45,8 +52,14 @@ data Estimate = Estimate {
     -- the confidence interval).
     , estConfidenceLevel :: {-# UNPACK #-} !Double
     -- ^ Confidence level of the confidence intervals.
-    } deriving (Eq, Show, Typeable, Data)
+    } deriving (Eq, Read, Show, Typeable, Data, Generic)
 
+instance FromJSON Estimate
+instance ToJSON Estimate
+
+instance Binary Estimate where
+    put (Estimate w x y z) = put w >> put x >> put y >> put z
+    get = Estimate <$> get <*> get <*> get <*> get
 instance NFData Estimate
 
 -- | Multiply the point, lower bound, and upper bound in an 'Estimate'
@@ -79,16 +92,18 @@ bootstrapBCA :: Double          -- ^ Confidence level
              -> [Estimator]     -- ^ Estimators
              -> [Resample]      -- ^ Resampled data
              -> [Estimate]
-bootstrapBCA confidenceLevel sample estimators resamples =
-    assert (confidenceLevel > 0 && confidenceLevel < 1)
-    runPar $ parMap (uncurry e) (zip estimators resamples)
+bootstrapBCA confidenceLevel sample estimators resamples
+  | confidenceLevel > 0 && confidenceLevel < 1
+      = runPar $ parMap (uncurry e) (zip estimators resamples)
+  | otherwise = error "Statistics.Resampling.Bootstrap.bootstrapBCA: confidence level outside (0,1) range"
   where
     e est (Resample resample)
-      | U.length sample == 1 = estimate pt pt pt confidenceLevel
-      | otherwise = 
+      | U.length sample == 1 || isInfinite bias =
+          estimate pt pt pt confidenceLevel
+      | otherwise =
           estimate pt (resample ! lo) (resample ! hi) confidenceLevel
       where
-        pt    = est sample
+        pt    = R.estimate est sample
         lo    = max (cumn a1) 0
           where a1 = bias + b1 / (1 - accel * b1)
                 b1 = bias + z1
