@@ -21,6 +21,7 @@ module Statistics.Sample
 
     -- * Statistics of location
     , mean
+    , welfordMean
     , meanWeighted
     , harmonicMean
     , geometricMean
@@ -54,11 +55,14 @@ module Statistics.Sample
     ) where
 
 import Statistics.Function (minMax)
+import Statistics.Sample.Internal (robustSumVar, sum)
 import Statistics.Types (Sample,WeightedSample)
+import qualified Data.Vector as V
 import qualified Data.Vector.Generic as G
+import qualified Data.Vector.Unboxed as U
 
 -- Operator ^ will be overriden
-import Prelude hiding ((^))
+import Prelude hiding ((^), sum)
 
 -- | /O(n)/ Range. The difference between the largest and smallest
 -- elements of a sample.
@@ -67,19 +71,31 @@ range s = hi - lo
     where (lo , hi) = minMax s
 {-# INLINE range #-}
 
+-- | /O(n)/ Arithmetic mean.  This uses Kahan-Babuška-Neumaier
+-- summation, so is more accurate than 'welfordMean' unless the input
+-- values are very large.
+mean :: (G.Vector v Double) => v Double -> Double
+mean xs = sum xs / fromIntegral (G.length xs)
+{-# SPECIALIZE mean :: U.Vector Double -> Double #-}
+{-# SPECIALIZE mean :: V.Vector Double -> Double #-}
+
 -- | /O(n)/ Arithmetic mean.  This uses Welford's algorithm to provide
 -- numerical stability, using a single pass over the sample data.
-mean :: (G.Vector v Double) => v Double -> Double
-mean = fini . G.foldl' go (T 0 0)
+--
+-- Compared to 'mean', this loses a surprising amount of precision
+-- unless the inputs are very large.
+welfordMean :: (G.Vector v Double) => v Double -> Double
+welfordMean = fini . G.foldl' go (T 0 0)
   where
     fini (T a _) = a
     go (T m n) x = T m' n'
         where m' = m + (x - m) / fromIntegral n'
               n' = n + 1
-{-# INLINE mean #-}
+{-# SPECIALIZE welfordMean :: U.Vector Double -> Double #-}
+{-# SPECIALIZE welfordMean :: V.Vector Double -> Double #-}
 
 -- | /O(n)/ Arithmetic mean for weighted sample. It uses a single-pass
--- algorithm analogous to the one used by 'mean'.
+-- algorithm analogous to the one used by 'welfordMean'.
 meanWeighted :: (G.Vector v (Double,Double)) => v (Double,Double) -> Double
 meanWeighted = fini . G.foldl' go (V 0 0)
     where
@@ -101,10 +117,7 @@ harmonicMean = fini . G.foldl' go (T 0 0)
 
 -- | /O(n)/ Geometric mean of a sample containing no negative values.
 geometricMean :: (G.Vector v Double) => v Double -> Double
-geometricMean = fini . G.foldl' go (T 1 0)
-  where
-    fini (T p n) = p ** (1 / fromIntegral n)
-    go (T p n) a = T (p * a) (n + 1)
+geometricMean = exp . mean . G.map log
 {-# INLINE geometricMean #-}
 
 -- | Compute the /k/th central moment of a sample.  The central moment
@@ -120,11 +133,12 @@ centralMoment a xs
     | a < 0  = error "Statistics.Sample.centralMoment: negative input"
     | a == 0 = 1
     | a == 1 = 0
-    | otherwise = G.sum (G.map go xs) / fromIntegral (G.length xs)
+    | otherwise = sum (G.map go xs) / fromIntegral (G.length xs)
   where
     go x = (x-m) ^ a
     m    = mean xs
-{-# INLINE centralMoment #-}
+{-# SPECIALIZE centralMoment :: Int -> U.Vector Double -> Double #-}
+{-# SPECIALIZE centralMoment :: Int -> V.Vector Double -> Double #-}
 
 -- | Compute the /k/th and /j/th central moments of a sample.
 --
@@ -142,7 +156,10 @@ centralMoments a b xs
         fini (V i j) = (i / n , j / n)
         m            = mean xs
         n            = fromIntegral (G.length xs)
-{-# INLINE centralMoments #-}
+{-# SPECIALIZE
+    centralMoments :: Int -> Int -> U.Vector Double -> (Double, Double) #-}
+{-# SPECIALIZE
+    centralMoments :: Int -> Int -> V.Vector Double -> (Double, Double) #-}
 
 -- | Compute the skewness of a sample. This is a measure of the
 -- asymmetry of its distribution.
@@ -169,7 +186,8 @@ centralMoments a b xs
 skewness :: (G.Vector v Double) => v Double -> Double
 skewness xs = c3 * c2 ** (-1.5)
     where (c3 , c2) = centralMoments 3 2 xs
-{-# INLINE skewness #-}
+{-# SPECIALIZE skewness :: U.Vector Double -> Double #-}
+{-# SPECIALIZE skewness :: V.Vector Double -> Double #-}
 
 -- | Compute the excess kurtosis of a sample.  This is a measure of
 -- the \"peakedness\" of its distribution.  A high kurtosis indicates
@@ -187,7 +205,8 @@ skewness xs = c3 * c2 ** (-1.5)
 kurtosis :: (G.Vector v Double) => v Double -> Double
 kurtosis xs = c4 / (c2 * c2) - 3
     where (c4 , c2) = centralMoments 4 2 xs
-{-# INLINE kurtosis #-}
+{-# SPECIALIZE kurtosis :: U.Vector Double -> Double #-}
+{-# SPECIALIZE kurtosis :: V.Vector Double -> Double #-}
 
 -- $variance
 --
@@ -205,11 +224,6 @@ kurtosis xs = c4 / (c2 * c2) - 3
 
 data V = V {-# UNPACK #-} !Double {-# UNPACK #-} !Double
 
-robustSumVar :: (G.Vector v Double) => Double -> v Double -> Double
-robustSumVar m samp = G.sum . G.map (square . subtract m) $ samp
-  where square x = x * x
-{-# INLINE robustSumVar #-}
-
 -- | Maximum likelihood estimate of a sample's variance.  Also known
 -- as the population variance, where the denominator is /n/.
 variance :: (G.Vector v Double) => v Double -> Double
@@ -218,7 +232,9 @@ variance samp
     | otherwise = 0
     where
       n = G.length samp
-{-# INLINE variance #-}
+{-# SPECIALIZE variance :: U.Vector Double -> Double #-}
+{-# SPECIALIZE variance :: V.Vector Double -> Double #-}
+
 
 -- | Unbiased estimate of a sample's variance.  Also known as the
 -- sample variance, where the denominator is /n/-1.
@@ -228,7 +244,8 @@ varianceUnbiased samp
     | otherwise = 0
     where
       n = G.length samp
-{-# INLINE varianceUnbiased #-}
+{-# SPECIALIZE varianceUnbiased :: U.Vector Double -> Double #-}
+{-# SPECIALIZE varianceUnbiased :: V.Vector Double -> Double #-}
 
 -- | Calculate mean and maximum likelihood estimate of variance. This
 -- function should be used if both mean and variance are required
@@ -240,25 +257,28 @@ meanVariance samp
     where
       n = G.length samp
       m = mean samp
-{-# INLINE meanVariance #-}
+{-# SPECIALIZE meanVariance :: U.Vector Double -> (Double,Double) #-}
+{-# SPECIALIZE meanVariance :: V.Vector Double -> (Double,Double) #-}
 
 -- | Calculate mean and unbiased estimate of variance. This
 -- function should be used if both mean and variance are required
 -- since it will calculate mean only once.
-meanVarianceUnb ::  (G.Vector v Double) => v Double -> (Double,Double)
+meanVarianceUnb :: (G.Vector v Double) => v Double -> (Double,Double)
 meanVarianceUnb samp
   | n > 1     = (m, robustSumVar m samp / fromIntegral (n-1))
   | otherwise = (m, 0)
     where
       n = G.length samp
       m = mean samp
-{-# INLINE meanVarianceUnb #-}
+{-# SPECIALIZE meanVarianceUnb :: U.Vector Double -> (Double,Double) #-}
+{-# SPECIALIZE meanVarianceUnb :: V.Vector Double -> (Double,Double) #-}
 
 -- | Standard deviation.  This is simply the square root of the
 -- unbiased estimate of the variance.
 stdDev :: (G.Vector v Double) => v Double -> Double
 stdDev = sqrt . varianceUnbiased
-{-# INLINE stdDev #-}
+{-# SPECIALIZE stdDev :: U.Vector Double -> Double #-}
+{-# SPECIALIZE stdDev :: V.Vector Double -> Double #-}
 
 robustSumVarWeighted :: (G.Vector v (Double,Double)) => v (Double,Double) -> V
 robustSumVarWeighted samp = G.foldl' go (V 0 0) samp
@@ -275,7 +295,8 @@ varianceWeighted samp
     | otherwise         = 0
     where
       fini (V s w) = s / w
-{-# INLINE varianceWeighted #-}
+{-# SPECIALIZE varianceWeighted :: U.Vector (Double,Double) -> Double #-}
+{-# SPECIALIZE varianceWeighted :: V.Vector (Double,Double) -> Double #-}
 
 -- $cancellation
 --
