@@ -25,15 +25,17 @@ module Statistics.Sample.KernelDensity
     -- $references
     ) where
 
-import Prelude hiding (const,min,max)
 import Numeric.MathFunctions.Constants (m_sqrt_2_pi)
-import Statistics.Function             (minMax, nextHighestPowerOfTwo)
-import Statistics.Math.RootFinding     (fromRoot, ridders)
-import Statistics.Sample.Histogram     (histogram_)
-import Statistics.Transform            (dct, idct)
+import Prelude hiding (const, min, max, sum)
+import Statistics.Function (minMax, nextHighestPowerOfTwo)
+import Statistics.Math.RootFinding (fromRoot, ridders)
+import Statistics.Sample.Histogram (histogram_)
+import Statistics.Sample.Internal (sum)
+import Statistics.Transform (dct, idct)
 import qualified Data.Vector.Generic as G
 import qualified Data.Vector.Unboxed as U
 
+
 -- | Gaussian kernel density estimator for one-dimensional data, using
 -- the method of Botev et al.
 --
@@ -53,6 +55,7 @@ kde n0 xs = kde_ n0 (lo - range / 10) (hi + range / 10) xs
   where
     (lo,hi) = minMax xs
     range   | U.length xs <= 1 = 1       -- Unreasonable guess
+            | lo == hi         = 1       -- All elements are equal
             | otherwise        = hi - lo
 
 -- | Gaussian kernel density estimator for one-dimensional data, using
@@ -74,7 +77,7 @@ kde_ :: Int
      -> U.Vector Double -> (U.Vector Double, U.Vector Double)
 kde_ n0 min max xs
   | U.null xs = error "Statistics.KernelDensity.kde: empty sample"
-  | n0 < 1    = error "Statistics.KernelDensity.kde: invalid number of points"
+  | n0 <= 1   = error "Statistics.KernelDensity.kde: invalid number of points"
   | otherwise = (mesh, density)
   where
     mesh = G.generate ni $ \z -> min + (d * fromIntegral z)
@@ -84,13 +87,13 @@ kde_ n0 min max xs
     !n  = fromIntegral ni
     !ni = nextHighestPowerOfTwo n0
     !r  = max - min
-    a   = dct . G.map (/ G.sum h) $ h
+    a   = dct . G.map (/ sum h) $ h
         where h = G.map (/ len) $ histogram_ ni min max xs
     !len    = fromIntegral (G.length xs)
     !t_star = fromRoot (0.28 * len ** (-0.4)) . ridders 1e-14 (0,0.1) $ \x ->
               x - (len * (2 * sqrt pi) * go 6 (f 7 x)) ** (-0.4)
       where
-        f q t = 2 * pi ** (q*2) * G.sum (G.zipWith g iv a2v)
+        f q t = 2 * pi ** (q*2) * sum (G.zipWith g iv a2v)
           where g i a2 = i ** q * a2 * exp ((-i) * sqr pi * t)
                 a2v = G.map (sqr . (*0.5)) $ G.tail a
                 iv = G.map sqr $ G.enumFromTo 1 (n-1)
